var _min_fish_speed = 3;
var _max_fish_speed = 5;
var _min_fish_move_duration = 10;
var _max_fish_move_duration = 15;
var Fish = cc.Sprite.extend({
	ctor: function() {
		this._super();
	},
	randomInt: function(min, max) {
		return min + Math.floor(Math.random() * (max - min + 1));
	},
	initData: function(imgArray, delegate) {
		this.imgArray = imgArray;
		this.delegate = delegate;
		this.setTexture(imgArray[0]);
		this.currentIndex = 0;
		var speedAdd = this.getFishConfigSpeed();
		this.speed = _min_fish_speed + Math.random() * (_max_fish_speed - _min_fish_speed);
		this.speed += speedAdd;
		this.angle = Math.random() * 360;
		var space = 10;
		while ((this.angle >= space && this.angle <= (180 - space)) || ((this.angle >= (180 + space) && this.angle <= (360 - space)))) {
			this.angle = Math.random() * 360;
		}
		var maxd = _max_fish_move_duration;
		var mind = _min_fish_move_duration;
		var duration = mind + (maxd - mind) * (1 - ((this.speed - _min_fish_speed) / (_max_fish_speed - _min_fish_speed)));
		this.moveDuration = duration;
	},
	addToFishPanel: function(fishPanel) {
		this.scale = fishPanel.nowScale;
		this.initPosition();
		fishPanel.addChild(this);
		this.setRotation (360 - this.angle + 180);
		if(this.angle > 90 && this.angle < 270) {
			this.setFlippedY(false);
		} else {
			this.setFlippedY(true);
		}
		this.updateSpeed(this.speed);
		this.startMove();
		cc.eventManager.addListener({
            event: cc.EventListener.TOUCH_ONE_BY_ONE,
            onTouchBegan: function(touch, event){
                var target = event.getCurrentTarget();
				var locationInNode = target.convertToNodeSpace(touch.getLocation());
				var s = target.getContentSize();
                var rect = cc.rect(0, 0, s.width, s.height);
                if (cc.rectContainsPoint(rect, locationInNode)) {
                	target.onTouched();
                	return true;
                } else {
                	return false;
                }
            }
        }, this);
	},
	startMove: function() {
		var tox;
		if (this.x <= 0) {
			// tox = cc.winSize.width + this.width * 2;
			tox = cc.winSize.width + this.width * this.scale / 2;
			var toy = this.y + cc.winSize.width * Math.tan(this.angle * (Math.PI * 2 / 360));
		}
		if (this.x >= cc.winSize.width) {
			// tox = 0 - this.width * 2;
			tox = 0 - this.width * this.scale / 2;
			var toy = this.y - cc.winSize.width * Math.tan(this.angle * (Math.PI * 2 / 360));
		}
		this.endPostion = cc.p(tox, toy);
		this.playMoveAction(this.moveDuration, cc.p(tox, toy));
	},
	playMoveAction: function(duration, toPoint) {
		this.runAction(
			cc.sequence(
				cc.moveTo(duration, toPoint),
				cc.callFunc(function (node) {
					node.removeFromParent();
					this.delegate.onOneFishRemoved();
				}, this)
				)
			);
	},
	initPosition: function() {
		var fishWidth = this.width * this.scale;
		// cc.log("fish width is " + fishWidth);
		var winWidth = cc.winSize.width;
		var initX = 0;
		var initY = 0;
		if (this.angle <= 90) {
			initX = 0 - fishWidth / 2;
			// initX = 0 - fishWidth / 2;
		} else if (this.angle <= 180) {
			initX = winWidth + fishWidth / 2;
			// initX = winWidth;
		} else if (this.angle <= 270) {
			initX = winWidth + fishWidth / 2;
			// initX = winWidth;
		} else {
			initX = 0 - fishWidth / 2;
			// initX = 0;
		};
		this.x = initX;
		// cc.log("x " + initX);
		var miny = cc.winSize.height * 0.1;
		var maxy = cc.winSize.height * 0.9;
		this.y = miny + Math.random() * (maxy - miny);
		// cc.log("x : " + this.x + " y : " + this.y);
		// cc.log("wx : " + cc.winSize.width + " wy : " + cc.winSize.height);
		this.startPosition = cc.p(this.x, this.y);
	},
	getFishConfigSpeed: function() {
		// var speedString = zg.AndroidUtils.getConfigString("fishSpeed", "Slow");
		// var speedAdd = 0;
		// if (speedString == "Normal") {
		// 	speedAdd = 1;
		// };
		// if (speedString == "Fast") {
		// 	speedAdd = 2;
		// };
		// return speedAdd;
		return 1;
	},
	onTouched: function() {
		this.updateSpeed(_max_fish_speed * 1.8);

		// update the move action
		this.stopAllActions();
		var nowPostion = this.getPosition();
		var duration = this.moveDuration * ((this.endPostion.x - nowPostion.x) / (this.endPostion.x - this.startPosition.x));
		this.playMoveAction(duration * 0.3, this.endPostion);
	},
	updateSpeed: function(newSpeed) {
		this.speed = newSpeed;
		this.unschedule(this.updateForAnimation);
		this.schedule(this.updateForAnimation, 0.25 / this.speed);
	},
	updateForAnimation: function() {
		this.currentIndex++;
		if (this.currentIndex >= this.imgArray.length) {
			this.currentIndex = 0;
		};
		this.setTexture(this.imgArray[this.currentIndex]);
	},
	onExit: function () {
        this._super();
        this.unscheduleAllCallbacks();
	},
});